]> git.rkrishnan.org Git - pihpsdr.git/commitdiff
Do not switch to low-latency in CW if the side tone volume is zero.
authorc vw <dl1ycf@darc.de>
Wed, 24 Feb 2021 16:08:27 +0000 (17:08 +0100)
committerc vw <dl1ycf@darc.de>
Wed, 24 Feb 2021 16:08:27 +0000 (17:08 +0100)
audio.c
portaudio.c

diff --git a/audio.c b/audio.c
index 4ec2f18152f1873fe623788a9ca58f30f1fd8172..a9bb39cda5298b6d053d7a552fd9a2d0263393f8 100644 (file)
--- a/audio.c
+++ b/audio.c
@@ -334,26 +334,31 @@ int cw_audio_write(float sample){
   g_mutex_lock(&rx->local_audio_mutex);
   if(rx->playback_handle!=NULL && rx->local_audio_buffer!=NULL) {
 
-    //
-    // For CW side tone, use short audio buffers
-    //
-    short_audio_buffer_size=rx->local_audio_buffer_size;
-    if (short_audio_buffer_size > 256) short_audio_buffer_size = 256;
-
-    if (rx->local_audio_cw == 0) {
+    if (rx->local_audio_cw == 0 && cw_keyer_sidetone_volume > 0) {
       //
       // first invocation of cw_audio_write e.g. after a RX/TX transition:
       // clear audio buffer local to pihpsdr, rewind ALSA buffer
       // if contains too many samples
       //
+      // if the keyer side tone volume is zero, then we need not care
+      // about side tone latency at all.
+      //
       rx->local_audio_cw=1;
       rx->local_audio_buffer_offset=0;
       if (snd_pcm_delay(rx->playback_handle, &delay) == 0) {
          if (delay > 1024) snd_pcm_rewind(rx->playback_handle, delay-1024);
       }
+      count=0;
     }
 
-    if (sample != 0.0) count=0;  // count upwards during silence
+    short_audio_buffer_size=rx->local_audio_buffer_size;
+    if (rx->local_audio_cw) {
+      //
+      // For CW side tone, use short audio buffers
+      //
+      if (short_audio_buffer_size > 256) short_audio_buffer_size = 256;
+      if (sample != 0.0) count=0;  // count upwards during silence
+    }
 
     //
     // Put sample into buffer
@@ -377,7 +382,7 @@ int cw_audio_write(float sample){
     }
     rx->local_audio_buffer_offset++;
 
-    if (++count >= 16) {
+    if (++count >= 16 && rx->local_audio_cw) {
       //
       // We have just seen 16 zero samples, so this is the right place
       // to adjust the buffer filling.
index 5fbb7bc552b624a7092f3d41fbc5db7424cbeb9e..dfb4e896d4b34436621cd1cae5c7cad0a33f212a 100644 (file)
@@ -59,7 +59,8 @@ int n_output_devices=0;
 // inserted. This usually only happens after TX/RX transitions
 //
 // If we go TX in CW mode, cw_audio_write() is called. If it is called for
// the first time, the ring buffer is cleared and only 256 samples of silence
 // are put into it. During the TX phase, the buffer filling remains low
 // which we need for small CW sidetone latencies. If we then go to RX again
 // a "low water mark" condition is detected in the first call to audio_write()
@@ -68,6 +69,8 @@ int n_output_devices=0;
 // Experiments indicate that we can indeed keep the ring buffer about half full
 // during RX and quite empty during CW-TX.
 //
+// If the sidetone volume is zero, the audio buffers are left unchanged
+//
 
 #define MY_AUDIO_BUFFER_SIZE 256
 #define MY_RING_BUFFER_SIZE  9600
@@ -584,7 +587,7 @@ int cw_audio_write(float sample) {
 
   g_mutex_lock(&rx->local_audio_mutex);
   if (rx->playback_handle != NULL && buffer != NULL) {
-    if (rx->local_audio_cw == 0) {
+    if (rx->local_audio_cw == 0 && cw_keyer_sidetone_volume > 0) {
       //
       // First time producing CW audio after RX/TX transition:
       // empty audio buffer and insert 512 samples of silence
@@ -593,21 +596,24 @@ int cw_audio_write(float sample) {
       bzero(buffer, 512*sizeof(float));
       rx->local_audio_buffer_inpt=512;
       rx->local_audio_buffer_outpt=0;
-    }
-    count++;
-    if (sample != 0.0) count=0;
-    adjust=0;
-    if (count >= 16) {
       count=0;
-      //
-      // We arrive here if we have seen 16 zero samples in a row.
-      // First look how many samples there are in the ring buffer
-      //
-      avail = rx->local_audio_buffer_inpt - rx->local_audio_buffer_outpt;
-      if (avail < 0) avail += MY_RING_BUFFER_SIZE;
-      if (avail > 768) adjust=2;  // too full: skip one sample
-      if (avail < 512) adjust=1;  // too empty: insert one sample
     }
+    adjust=0;
+    if (rx->local_audio_cw) {
+      count++;
+      if (sample != 0.0) count=0;
+      if (count >= 16) {
+        count=0;
+        //
+        // We arrive here if we have seen 16 zero samples in a row.
+        // First look how many samples there are in the ring buffer
+        //
+        avail = rx->local_audio_buffer_inpt - rx->local_audio_buffer_outpt;
+        if (avail < 0) avail += MY_RING_BUFFER_SIZE;
+        if (avail > 768) adjust=2;  // too full: skip one sample
+        if (avail < 512) adjust=1;  // too empty: insert one sample
+      }
+    }  
     switch (adjust) {
       case 0:
         //
@@ -59,7 +59,8 @@ int n_output_devices=0;
 //
 // If we go TX in CW mode, cw_audio_write() is called. If it is called for
-// the first time, the ring buffer is cleared and only 256 samples of silence
+// the first time with a non-zero sidetone volume,
+// the ring buffer is cleared and only 256 samples of silence
 // are put into it. During the TX phase, the buffer filling remains low
 // which we need for small CW sidetone latencies. If we then go to RX again